#pragma once

#include <lvgl/src/lv_core/lv_obj.h>
#include <chrono>
#include <cstdint>
#include <memory>
#include <displayapp/Controllers.h>
#include "displayapp/screens/Screen.h"
#include "components/datetime/DateTimeController.h"
#include "utility/DirtyValue.h"

namespace Pinetime {
  namespace Controllers {
    class Settings;
    class Battery;
    class Ble;
    class NotificationManager;
    class HeartRateController;
    class MotionController;
  }

  struct CalendarEntry {
    char* name;
    uint32_t duration;
  };

  struct TaskEntry {
    char* name;
    uint32_t priority;
  };

  namespace Applications {
    namespace Screens {

      class WatchFaceTerminal : public Screen {
      public:
        WatchFaceTerminal(Controllers::DateTime& dateTimeController,
                          const Controllers::Battery& batteryController,
                          const Controllers::Ble& bleController,
                          Controllers::NotificationManager& notificationManager,
                          Controllers::Settings& settingsController,
                          Controllers::HeartRateController& heartRateController,
                          Controllers::MotionController& motionController);
        ~WatchFaceTerminal() override;

        void Refresh() override;

      private:
        Utility::DirtyValue<int> batteryPercentRemaining {};
        Utility::DirtyValue<bool> powerPresent {};
        Utility::DirtyValue<bool> bleState {};
        Utility::DirtyValue<bool> bleRadioEnabled {};
        Utility::DirtyValue<std::chrono::time_point<std::chrono::system_clock, std::chrono::seconds>> currentDateTime {};
        Utility::DirtyValue<uint32_t> stepCount {};
        Utility::DirtyValue<uint8_t> heartbeat {};
        Utility::DirtyValue<bool> heartbeatRunning {};
        Utility::DirtyValue<bool> notificationState {};
        Utility::DirtyValue<std::chrono::time_point<std::chrono::system_clock, std::chrono::days>> currentDate;

        CalendarEntry fixedtime[14] = {
          {"Present FuckTime", 30},
          {"Party @ David's", 3*60},
          {"Catch the L Train", 4*60},
          {"Sleep", 8*60},
          {"Wake up", 30},
          {"Work", 4*60},
          {"Eat Lunch", 60},
          {"Afternoon Work", 4*60},
          {"Get groceries", 20},
          {"Prepare dinner", 30},
          {"Eat", 60},
          {"Sleep", 8*60},
          {"Rave", 30},
          {"Repeat", 10}
        };
        TaskEntry flextime[5] = {
          {"Enjoy the sunset", 1},
          // {"Enjoy the crisp night air", 3},
          // {"Savor the moment", 5},
          {"Collect leaves", 7},
          {"Read southern gothic", 9},
          {"Attend Food Not Bombs", 11},
          {"Undermine authority", 13},
        };
        TaskEntry season[1] = {
          {"大雪 Winter begins", 0},
        };
        TaskEntry actualization[2] = {
          {"Become a tea monk", 1},
          {"Organize BuyNothing", 3},
        };
        TaskEntry community[2] = {
          {"Publish zines", 1},
          {"Cultivate the next", 3},
        };

        lv_obj_t* fixedtime_labels[sizeof(fixedtime)/sizeof(CalendarEntry)];
        uint32_t calIndex = 0;
        uint32_t currentDuration = 0;

        lv_obj_t* now_line;
        lv_obj_t* batteryValue;
        lv_obj_t* notificationIcon;

        Controllers::DateTime& dateTimeController;
        const Controllers::Battery& batteryController;
        const Controllers::Ble& bleController;
        Controllers::NotificationManager& notificationManager;
        Controllers::Settings& settingsController;
        Controllers::HeartRateController& heartRateController;
        Controllers::MotionController& motionController;

        lv_task_t* taskRefresh;
      };
    }

    template <>
    struct WatchFaceTraits<WatchFace::Terminal> {
      static constexpr WatchFace watchFace = WatchFace::Terminal;
      static constexpr const char* name = "Terminal";

      static Screens::Screen* Create(AppControllers& controllers) {
        return new Screens::WatchFaceTerminal(controllers.dateTimeController,
                                              controllers.batteryController,
                                              controllers.bleController,
                                              controllers.notificationManager,
                                              controllers.settingsController,
                                              controllers.heartRateController,
                                              controllers.motionController);
      };

      static bool IsAvailable(Pinetime::Controllers::FS& /*filesystem*/) {
        return true;
      }
    };
  }
}
